//---------------------------------------------------------------------------
#ifndef IvorySurfaceH
#define IvorySurfaceH

#include "IvoryDraw.h"
//---------------------------------------------------------------------------
class TIvoryDraw;

class TIvoryMapper;

enum TSurfaceType { stUnspecified, stPrimary, stBack, stOffScreen };

typedef void __fastcall (__closure* TGetMapEvent)(System::TObject* Sender,
                         int Row, int Col, int &MapIndex);

class IVORY_package TIvorySurface : public TComponent
{
	friend TIvoryDraw;
   friend TIvoryCanvas;
private:
	bool FAutoCreate;
	LPDIRECTDRAWSURFACE FDirectDrawSurface;
   LPDIRECTDRAWCLIPPER FDirectDrawClipper;
   TSurfaceType FSurfaceType;
   int FWidth;
   int FHeight;
   bool FTransparent;
   int FTransparentColor;
   TIvoryCanvas* FCanvas;
   TIvoryBitmap* FBackgroundBitmap;
   bool FDrawBackground;
   bool FTileBackground;
   TColor FBackgroundColor;
   bool FClearBackground;
   TIvoryMapper* FMapper;
   int FMapLeft;
   int FMapTop;
   int FMapColumns;
   int FMapRows;
   bool FMapEnabled;
   int* FMap;

   Classes::TNotifyEvent FBeforePaint;
   Classes::TNotifyEvent FOnPaint;
   Classes::TNotifyEvent FAfterPaint;
   Classes::TNotifyEvent FAfterDrawMap;
   Classes::TNotifyEvent FOnCreate;
   Classes::TNotifyEvent FOnDestroy;
   Classes::TNotifyEvent FOnChange;
   TGetMapEvent FOnGetMap;
protected:
   void __fastcall BeforeDestruction();
	void __fastcall CreatePrimary();
   void __fastcall CreateBack(TIvorySurface* primary);
   void __fastcall _CreateOffScreen(); // FWidth and FHeight must be set!
   void __fastcall Release();
   void __fastcall SetAutoCreate(bool ac);
   void __fastcall SetWidth(int width);
   void __fastcall SetHeight(int height);
   void __fastcall SetTransparent(bool transparent);
   void __fastcall SetTransparentColor(int tcolor);
//   void __fastcall SetBackgroundBitmap(TIvoryBitmap* bitmap);
//   void __fastcall SetDrawBackground(bool drawbkg);
//   void __fastcall SetTileBackground(bool tilebkg);
//   void __fastcall SetBackgroundColor(TColor bkcolor);
   void __fastcall GrabCanvas();
   void __fastcall ReleaseCanvas();
   inline void __fastcall Restore() { DirectDrawSurface->Restore(); }
//   void __fastcall SetMapEnabled(bool enabled);
   void __fastcall DrawMap();
   void __fastcall CreateMapperSurface();
public:
	__fastcall TIvorySurface(Classes::TComponent* AOwner);
   __fastcall ~TIvorySurface();
   void __fastcall CreateOffScreen();
   void __fastcall RecreateOffScreen(int width, int height);
   void __fastcall RecreateOffScreen(int width, int height, bool transparent, int color = 0);
   void __fastcall CopySurface(int origx, int origy, TIvorySurface* surface);
   void __fastcall CopySurface(int origx, int origy, TIvorySurface* surface, const TRect rect);
   void __fastcall Paint();

	__property LPDIRECTDRAWSURFACE DirectDrawSurface = { read=FDirectDrawSurface };
   __property LPDIRECTDRAWCLIPPER DirectDrawClipper = { read=FDirectDrawClipper };
   __property TSurfaceType SurfaceType = { read=FSurfaceType };
   __property TIvoryCanvas* Canvas = { read=FCanvas };
   __property int* Map = { read=FMap, write=FMap };
__published:
	__property bool AutoCreate = { read=FAutoCreate, write=SetAutoCreate, default=true };
	__property int Width = { read=FWidth, write=SetWidth, default=640 };
	__property int Height = { read=FHeight, write=SetHeight, default=480 };
   __property bool Transparent = { read=FTransparent, write=SetTransparent, default=false };
   __property int TransparentColor = { read=FTransparentColor, write=SetTransparentColor, default=0 };
   __property TIvoryBitmap* BackgroundBitmap = { read=FBackgroundBitmap, write=FBackgroundBitmap, default=NULL };
   __property bool DrawBackground = { read=FDrawBackground, write=FDrawBackground, default=false };
   __property bool TileBackground = { read=FTileBackground, write=FTileBackground, default=false };
   __property TColor BackgroundColor = { read=FBackgroundColor, write=FBackgroundColor, default=clBlack };
   __property bool ClearBackground = { read=FClearBackground, write=FClearBackground, default=true };
   __property TIvoryMapper* Mapper = { read=FMapper, write=FMapper, default=NULL };
   __property int MapLeft = { read=FMapLeft, write=FMapLeft, default=0 };
   __property int MapTop = { read=FMapTop, write=FMapTop, default=0 };
   __property int MapColumns = { read=FMapColumns, write=FMapColumns, default=0 };
   __property int MapRows = { read=FMapRows, write=FMapRows, default=0 };
   __property bool MapEnabled = { read=FMapEnabled, write=FMapEnabled, default=false };

   __property Classes::TNotifyEvent BeforePaint = { read=FBeforePaint, write=FBeforePaint, default=NULL };
   __property Classes::TNotifyEvent OnPaint = { read=FOnPaint, write=FOnPaint, default=NULL };
   __property Classes::TNotifyEvent AfterPaint = { read=FAfterPaint, write=FAfterPaint, default=NULL };
   __property Classes::TNotifyEvent AfterDrawMap = { read=FAfterDrawMap, write=FAfterDrawMap, default=NULL };
   __property Classes::TNotifyEvent OnCreate = { read=FOnCreate, write=FOnCreate, default=NULL };
   __property Classes::TNotifyEvent OnDestroy = { read=FOnDestroy, write=FOnDestroy, default=NULL };
   __property Classes::TNotifyEvent OnChange = { read=FOnChange, write=FOnChange, default=NULL };
   __property TGetMapEvent OnGetMap = { read=FOnGetMap, write=FOnGetMap, default=NULL };
};
//---------------------------------------------------------------------------
#endif
